library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity EROM128b is
    Port ( clk : in std_logic;
           rstn, tcp_establish : in  STD_LOGIC;
           loc_addr : in  STD_LOGIC_VECTOR (31 downto 0);
           loc_we : in  STD_LOGIC;
           loc_wd : in  STD_LOGIC_VECTOR (7 downto 0);
           loc_re : in  STD_LOGIC;
           loc_ack : out  STD_LOGIC;
           loc_rd : out  STD_LOGIC_VECTOR (7 downto 0);

	genkp, genclk, nkp2, clkp2 : in std_logic;
	kpout, kpout2 : out std_logic;
	UDPSEL : out std_logic; 
	UDPWE : out std_logic; 
	UDPREAD : out std_logic; 
 	UDPADD  : out std_logic_vector(7 downto 0); 
 	UDPWDT  : out std_logic_vector(15 downto 0); 
 	UDPRDT  : in std_logic_vector(15 downto 0); 

	rw : in std_logic;
	begf : in std_logic;
	endf, rce : out std_logic;
	add : out std_logic_vector(5 downto 0);
	di : in std_logic_vector(15 downto 0);
	do : out std_logic_vector(15 downto 0);
	derr : out std_logic;

	tmce : out std_logic;
	tmadd : out std_logic_vector(2 downto 0);
	tmdi : out std_logic_vector(15 downto 0);
	tmdo : in std_logic_vector(15 downto 0);

	daadd : in STD_LOGIC_VECTOR(2 downto 0);
	emdt : out STD_LOGIC_VECTOR(63 downto 0);

	tdc_clock : out STD_LOGIC_VECTOR(55 downto 0);	--for TDC timestamp, 2012 2/10

	ptlldtm : out std_logic_vector(63 downto 0);	--lld=(59:48), time_hi=(47:24), time_low(23:0)

	ii_read : out std_logic;
	ii_di : in std_logic_vector(255 downto 0);
	ii_do : out std_logic_vector(255 downto 0);

--	mb : inout std_logic_vector(3 downto 0)); --mb(3)=cs, mb(2)=sk, mb(1)=di, mb(0)=do
	mcs, msk, mdo : out std_logic;
	mdi : in std_logic);
end EROM128b;

architecture Behavioral of EROM128b is
component EROM46b_rw
    Port ( clk : in std_logic;
	rw : in std_logic;
	begf : in std_logic;
	endf, busy : out std_logic;
	cm : in std_logic_vector(9 downto 0);
	di : in std_logic_vector(7 downto 0);
	do : out std_logic_vector(7 downto 0);
--	mb : inout std_logic_vector(3 downto 0)); --mb(3)=cs, mb(2)=sk, mb(1)=di, mb(0)=do
	mcs, msk, mdo : out std_logic;
	mdi : in std_logic);
end component;
component SRAM16d
    Port ( clk : in std_logic;
	we : in std_logic;
	add : in std_logic_vector(3 downto 0);
	di : in std_logic_vector(15 downto 0);
	do : out std_logic_vector(15 downto 0);
	dadd : in std_logic_vector(3 downto 0);
	ddo : out std_logic_vector(15 downto 0));
end component;
component kp_gen2
    Port ( clk : in std_logic;
	caddr : in std_logic_vector(7 downto 0);		--crate address
	t0tm : in std_logic_vector(39 downto 0);		--max time bin
	ptclock : in std_logic_vector(55 downto 0);		--real clock
	nkp : in std_logic;
	rkp : out std_logic);
end component;
	signal vmef, acc, dacc, ddacc, rwf, bf, ef, def, prce, wrce, pendf, dlwe : STD_LOGIC;
	signal adct: STD_LOGIC_VECTOR(7 downto 0);	--64 words address
	signal cm: STD_LOGIC_VECTOR(9 downto 0);	--buffer of the command
	signal mdib, mdob, bmdi: STD_LOGIC_VECTOR(7 downto 0);	--buffer of the data
--	signal tsdata: STD_LOGIC_VECTOR(7 downto 0);	--buffer of the data

	signal up_1h, uph_on, emrw, infwea, infweb, infwec, infwed, dbf, locwb, lwtb, busy, ptmce : STD_LOGIC;
	signal ii_ct: STD_LOGIC_VECTOR(4 downto 0);	--32 bytes address
	signal hourct: STD_LOGIC_VECTOR(5 downto 0);	--alive time
	signal atime_ct, atime: STD_LOGIC_VECTOR(31 downto 0);	--alive time
	signal at_ct: STD_LOGIC_VECTOR(1 downto 0);	--32 bytes address
	signal lwe_ct: STD_LOGIC_VECTOR(3 downto 0);
	signal reset_ct: STD_LOGIC_VECTOR(3 downto 0);
	signal padd: STD_LOGIC_VECTOR(5 downto 0);
	signal ramadd: STD_LOGIC_VECTOR(5 downto 0);
	signal pdi, pdo, infdi, infdib, infdo, infdoa, infdob, infdoc, infdod, erwtct: STD_LOGIC_VECTOR(15 downto 0);
	signal em_ad: STD_LOGIC_VECTOR(3 downto 0);	--as emdt address
	signal wait_ct : STD_LOGIC_VECTOR(23 downto 0);

	signal dgkp, ddgkp, dgclk, ddgclk, ptsetpi, ptsetpi2, ptsetrck, ptsetck, dgkp2, ddgkp2 : STD_LOGIC;
	signal ptstatus : STD_LOGIC_VECTOR(15 downto 0);
	signal ptcrate : STD_LOGIC_VECTOR(7 downto 0);
	signal ptpulseid, ptsetpulseid, ptpulseid2, ptsetpulseid2 : STD_LOGIC_VECTOR(39 downto 0);
	signal ptclock, ptsetclock : STD_LOGIC_VECTOR(55 downto 0);
	signal ptsetlldtm : STD_LOGIC_VECTOR(63 downto 0);
		--ptlldtm(59:48), ptlldtm(47:24), ptlldtm(23:0)
	signal dawe : STD_LOGIC;
	signal tim_1m : STD_LOGIC;
	signal tim_1m_ct : STD_LOGIC_VECTOR(15 downto 0);

begin
U10: kp_gen2 port map(clk, ptcrate, ptpulseid, ptclock, dgkp, kpout);
U11: kp_gen2 port map(clk, ptcrate, ptpulseid2, ptclock, dgkp2, kpout2);

U0: EROM46b_rw port map(clk, rwf, bf, ef, busy, cm, mdib, mdob, mcs, msk, mdo, mdi);
rce <= prce when vmef='1' else '0';
pdi <= di when vmef='1' else infdo;
endf <= pendf when begf='1' else '0';
add <= padd;
ii_read <= '1' when acc='1' and vmef='0' else '0';
do <= pdo;
--u5: RAMB16_S9 port map(clk=>clk, we=>wwe, addr=>loc_addr(10 downto 0), di=>loc_wd, do=>loc_rd, en=>'1', ssr=>not rstn, dip=>"0");
--wwe <= lowe and not dlowe;
em_ad <= "0"&daadd;
U3: SRAM16d port map(clk, infwea, ramadd(5 downto 2), infdi, infdoa, em_ad, emdt(63 downto 48));
U4: SRAM16d port map(clk, infweb, ramadd(5 downto 2), infdi, infdob, em_ad, emdt(47 downto 32));
U5: SRAM16d port map(clk, infwec, ramadd(5 downto 2), infdi, infdoc, em_ad, emdt(31 downto 16));
U6: SRAM16d port map(clk, infwed, ramadd(5 downto 2), infdi, infdod, em_ad, emdt(15 downto 0));
infdi <= pdo when acc='1' else infdib;
infwea <= prce when vmef='0' and ramadd(1 downto 0)="00" else '0';
infweb <= prce when vmef='0' and ramadd(1 downto 0)="01" else '0';
infwec <= prce when vmef='0' and ramadd(1 downto 0)="10" else '0';
infwed <= prce when vmef='0' and ramadd(1 downto 0)="11" else '0';
infdo <= infdoa when ramadd(1 downto 0)="00" else
	infdob when ramadd(1 downto 0)="01" else
	infdoc when ramadd(1 downto 0)="10"	else	infdod;
ramadd <= padd when acc='1' else loc_addr(6 downto 1);
tmadd <= loc_addr(3 downto 1);
derr <= acc;
--U15: SRAM64 port map(clk, dawe, loc_addr(6 downto 1), dawdt, dardt);
udpadd <= loc_addr(8 downto 1) when loc_addr(31 downto 9)="00000000000000000000001" else
	"00000000";	--0x200~3ff, dac setting area
udpwe <= dawe;
udpread <= loc_re when loc_addr(31 downto 9)="00000000000000000000001" else '0';
udpsel <= '1' when loc_addr(31 downto 9)="00000000000000000000001" else '0';
TDC_clock <= ptclock;	--for TDC timestamp, 2012 2/10

process (CLK) 
begin
if CLK='1' and CLK'event then
	tmce <= ptmce;
	dacc <= acc;	ddacc<=dacc;
	dgkp<=genkp;	ddgkp<=dgkp;
	dgkp2<=nkp2;	ddgkp2<=dgkp2;
	dgclk<=genclk;	ddgclk<=dgclk;
	loc_ack <= loc_we or loc_re;
	if ptsetpi='1' then
		ptpulseid <= ptsetpulseid;
		ptsetpi <= '0';
	elsif dgkp='0' and ddgkp='1' then
		ptpulseid <= ptpulseid+1;
	end if; 
	if ptsetpi2='1' then
		ptpulseid2 <= ptsetpulseid2;
		ptsetpi2 <= '0';
	elsif clkp2='1' then
		ptpulseid2 <= (others=>'0');
	elsif dgkp2='0' and ddgkp2='1' then
		ptpulseid2 <= ptpulseid2+1;
	end if;
	dlwe <= loc_we;
--	if rstn='1' then
	if reset_ct/="0101" then
		ptlldtm <= X"0100000000000000";	--lld=(59:48), time_hi=(47:24), time_low(23:0)
		ptclock <= (others=>'0');
		ptpulseid2 <= (others=>'0');
	elsif ptsetrck='1' then
		ptlldtm <= ptsetlldtm;
		ptsetrck <= '0';
	elsif ptsetck='1' then
		ptclock <= ptsetclock;
		ptsetck <= '0';
	elsif dgclk='1' and ddgclk='0' then
--		ptclock(10 downto 0) <= (others=>'0');
--		ptclock(55 downto 11) <= ptclock(55 downto 11)+1;
		ptclock(17 downto 0) <= (others=>'0');			--for 256Hz
		ptclock(55 downto 18) <= ptclock(55 downto 18)+1;	--for 256Hz
	else
--		ptclock(10 downto 0) <= ptclock(10 downto 0)+1;
		ptclock(17 downto 0) <= ptclock(17 downto 0)+1;	--for 256Hz
	end if;

	if prce='1' then		prce <= '0';	end if; 
	if ptmce='1' then		ptmce <= '0';	end if; 
	if dawe='1' then	dawe<='0';	end if;
	if reset_ct/="0101" then
		reset_ct <= reset_ct+1;
		up_1h <= '0';
		uph_on <= '0';
		ii_ct <= (others=>'0');
		locwb <= '0';

		acc <= '1';
		def <= '0';
		vmef <= '0';
		bf <= '1';
		emrw <= '0';
		rwf <= '0';
		wait_ct <= (others=>'0');
		adct <= (others=>'0');
		padd <= (others=>'0');
		cm <= "1101000000";	--READ behind the half of EROM
	elsif loc_we='1' and dlwe='0' then
		if loc_addr(31 downto 7)="000000000000000000000000" and acc='0' and uph_on='0' then
			if loc_addr(0)='0' then
				infdib(15 downto 8) <= loc_wd;
			else
				infdib(7 downto 0) <= loc_wd;
				prce <= '1';
			end if;
		elsif loc_addr=X"00000100" and loc_wd(7 downto 1)="0101101" then	--0x5a or 0x5b
							--write 'Z' at address=0x100 >> write EROM
			if acc='0' and begf='0' and uph_on='0' and up_1h='0' then	--0x5a or 0x5b
				acc <= '1';
				vmef <= '0';
				bf <= '1';
				emrw <= loc_wd(0);	--0x5a is read, 0x5b is write.
				rwf <= '0';
				adct <= (others=>'0');
				padd <= (others=>'0');
				wait_ct <= (others=>'0');
				if loc_wd(0)='1' then
					cm <= "1001100000";	--EWEN
				else
					cm <= "1100000000";	--READ EROM
				end if;
			else
				locwb <= '1';
				lwtb <= loc_wd(0);
			end if;
		elsif loc_addr(31 downto 3)="0000000000000000000000110000" then	--0x180~187
			if loc_addr(0)='0' then
				tmdi(15 downto 8) <= loc_wd;
			else
				tmdi(7 downto 0) <= loc_wd;
				ptmce <= '1';
			end if;
		elsif loc_addr(31 downto 3)="0000000000000000000000110001" then	--0x188~18f
			case loc_addr(2 downto 0) is
			when "000"=>	ptstatus(15 downto 8) <= loc_wd;
			when "001"=>	ptstatus(7 downto 0) <= loc_wd;
			when "010"=>	ptcrate <= loc_wd;
			when "011"=>	ptsetpulseid(39 downto 32) <= loc_wd;
			when "100"=>	ptsetpulseid(31 downto 24) <= loc_wd;
			when "101"=>	ptsetpulseid(23 downto 16) <= loc_wd;
			when "110"=>	ptsetpulseid(15 downto 8) <= loc_wd;
			when others=>	ptsetpulseid(7 downto 0) <= loc_wd;
								ptsetpi <= '1';
			end case;
		elsif loc_addr(31 downto 3)="0000000000000000000000110010" then	--0x190~197
			case loc_addr(2 downto 0) is
			when "000"=>	ptsetclock(55 downto 48) <= loc_wd;
			when "001"=>	ptsetclock(47 downto 40) <= loc_wd;
			when "010"=>	ptsetclock(39 downto 32) <= loc_wd;
			when "011"=>	ptsetclock(31 downto 24) <= loc_wd;
			when "100"=>	ptsetclock(23 downto 16) <= loc_wd;
			when "101"=>	ptsetclock(15 downto 8) <= loc_wd;
			when "110"=>	ptsetclock(7 downto 0) <= loc_wd;
								ptsetck <= '1';
			when others=>
			end case;
		elsif loc_addr(31 downto 3)="0000000000000000000000110011" then	--0x198~19f
		  if tcp_establish='0' then
			case loc_addr(2 downto 0) is
			when "000"=>	ptsetlldtm(63 downto 56) <= loc_wd;
			when "001"=>	ptsetlldtm(55 downto 48) <= loc_wd;
			when "010"=>	ptsetlldtm(47 downto 40) <= loc_wd;
			when "011"=>	ptsetlldtm(39 downto 32) <= loc_wd;
			when "100"=>	ptsetlldtm(31 downto 24) <= loc_wd;
			when "101"=>	ptsetlldtm(23 downto 16) <= loc_wd;
			when "110"=>	ptsetlldtm(15 downto 8) <= loc_wd;
			when others=>	ptsetlldtm(7 downto 0) <= loc_wd;
								ptsetrck <= '1';
			end case;
		  end if;
		elsif loc_addr(31 downto 3)="0000000000000000000000110100" then	--0x1a0~1a7
			case loc_addr(2 downto 0) is
			when "000"=>	ptsetpulseid2(39 downto 32) <= loc_wd;
			when "001"=>	ptsetpulseid2(31 downto 24) <= loc_wd;
			when "010"=>	ptsetpulseid2(23 downto 16) <= loc_wd;
			when "011"=>	ptsetpulseid2(15 downto 8) <= loc_wd;
			when "100"=>	ptsetpulseid2(7 downto 0) <= loc_wd;
								ptsetpi2 <= '1';
			when others=>
			end case;
		elsif loc_addr(31 downto 9)="0000000000000000000001" then	--0x200~3ff, dac setting area
			if loc_addr(0)='0' then
				udpwdt(15 downto 8) <= loc_wd;
			else
				udpwdt(7 downto 0) <= loc_wd;
				dawe <= '1';
			end if;
		end if;
	elsif loc_re='1' then
		if loc_addr(31 downto 7)="000000000000000000000000" then
			if loc_addr(0)='0' then
				loc_rd <= infdo(15 downto 8);
			else
				loc_rd <= infdo(7 downto 0);
			end if;
		elsif loc_addr(31 downto 3)="0000000000000000000000110000" then	--0x180~187
			if loc_addr(0)='0' then
				loc_rd <= tmdo(15 downto 8);
			elsif loc_addr(2 downto 1)="00" then
				loc_rd <= tmdo(7 downto 6)&tcp_establish&tmdo(4 downto 0);	--2008 12/17
			else
				loc_rd <= tmdo(7 downto 0);
			end if;
		elsif loc_addr(31 downto 3)="0000000000000000000000110001" then	--0x188~18f
			case loc_addr(2 downto 0) is
			when "000"=>	loc_rd <= ptstatus(15 downto 8);
			when "001"=>	loc_rd <= ptstatus(7 downto 0);
			when "010"=>	loc_rd <= ptcrate;
			when "011"=>	loc_rd <= ptpulseid(39 downto 32);
			when "100"=>	loc_rd <= ptpulseid(31 downto 24);
			when "101"=>	loc_rd <= ptpulseid(23 downto 16);
			when "110"=>	loc_rd <= ptpulseid(15 downto 8);
			when others=>	loc_rd <= ptpulseid(7 downto 0);
			end case;
		elsif loc_addr(31 downto 3)="0000000000000000000000110010" then	--0x190~197	--0x198~19f
			case loc_addr(2 downto 0) is
			when "000"=>	loc_rd <= ptclock(55 downto 48);
			when "001"=>	loc_rd <= ptclock(47 downto 40);
			when "010"=>	loc_rd <= ptclock(39 downto 32);
			when "011"=>	loc_rd <= ptclock(31 downto 24);
			when "100"=>	loc_rd <= ptclock(23 downto 16);
			when "101"=>	loc_rd <= ptclock(15 downto 8);
			when "110"=>	loc_rd <= ptclock(7 downto 0);
			when others=>	loc_rd <= (others=>'0');
			end case;
		elsif loc_addr(31 downto 3)="0000000000000000000000110011" then	--0x198~19f	--0x190~197
			case loc_addr(2 downto 0) is
			when "000"=>	loc_rd <= ptsetlldtm(63 downto 56);
			when "001"=>	loc_rd <= ptsetlldtm(55 downto 48);
			when "010"=>	loc_rd <= ptsetlldtm(47 downto 40);
			when "011"=>	loc_rd <= ptsetlldtm(39 downto 32);
			when "100"=>	loc_rd <= ptsetlldtm(31 downto 24);
			when "101"=>	loc_rd <= ptsetlldtm(23 downto 16);
			when "110"=>	loc_rd <= ptsetlldtm(15 downto 8);
			when others=>	loc_rd <= ptsetlldtm(7 downto 0);
			end case;
		elsif loc_addr(31 downto 3)="0000000000000000000000110100" then	--0x1a0~1a7
			case loc_addr(2 downto 0) is
			when "000"=>	loc_rd <= ptpulseid2(39 downto 32);
			when "001"=>	loc_rd <= ptpulseid2(31 downto 24);
			when "010"=>	loc_rd <= ptpulseid2(23 downto 16);
			when "011"=>	loc_rd <= ptpulseid2(15 downto 8);
			when "100"=>	loc_rd <= ptpulseid2(7 downto 0);
			when others=>	loc_rd <= (others=>'0');
			end case;
		elsif loc_addr(31 downto 9)="0000000000000000000001" then	--0x200~3ff, dac setting area
			if loc_addr(0)='0' then
				loc_rd <= udprdt(15 downto 8);
			else
				loc_rd <= udprdt(7 downto 0);
			end if;
		else
			case loc_addr(4 downto 0) is
			when	"00000"=>	loc_rd<=ii_di(255 downto 248);
			when	"00001"=>	loc_rd<=ii_di(247 downto 240);
			when	"00010"=>	loc_rd<=ii_di(239 downto 232);
			when	"00011"=>	loc_rd<=ii_di(231 downto 224);
			when	"00100"=>	loc_rd<=ii_di(223 downto 216);
			when	"00101"=>	loc_rd<=ii_di(215 downto 208);
			when	"00110"=>	loc_rd<=ii_di(207 downto 200);
			when	"00111"=>	loc_rd<=ii_di(199 downto 192);
			when	"01000"=>	loc_rd<=ii_di(191 downto 184);
			when	"01001"=>	loc_rd<=ii_di(183 downto 176);
			when	"01010"=>	loc_rd<=ii_di(175 downto 168);
			when	"01011"=>	loc_rd<=ii_di(167 downto 160);
			when	"01100"=>	loc_rd<=ii_di(159 downto 152);
			when	"01101"=>	loc_rd<=ii_di(151 downto 144);
			when	"01110"=>	loc_rd<=ii_di(143 downto 136);
			when	"01111"=>	loc_rd<=ii_di(135 downto 128);
			when	"10000"=>	loc_rd<=ii_di(127 downto 120);
			when	"10001"=>	loc_rd<=ii_di(119 downto 112);
			when	"10010"=>	loc_rd<=ii_di(111 downto 104);
			when	"10011"=>	loc_rd<=ii_di(103 downto 96);
			when	"10100"=>	loc_rd<=ii_di(95 downto 88);
			when	"10101"=>	loc_rd<=ii_di(87 downto 80);
			when	"10110"=>	loc_rd<=ii_di(79 downto 72);
			when	"10111"=>	loc_rd<=ii_di(71 downto 64);
			when	"11000"=>	loc_rd<=ii_di(63 downto 56);
			when	"11001"=>	loc_rd<=ii_di(55 downto 48);
			when	"11010"=>	loc_rd<=ii_di(47 downto 40);
			when	"11011"=>	loc_rd<=ii_di(39 downto 32);
			when	"11100"=>	loc_rd<=ii_di(31 downto 24);
			when	"11101"=>	loc_rd<=ii_di(23 downto 16);
			when	"11110"=>	loc_rd<=ii_di(15 downto 8);
			when	others=>	loc_rd<=ii_di(7 downto 0);
			end case;
		end if;
	elsif acc='1' then
		if ef='1' and def='0' then
			def <= '1';
			bf <= '0';
		elsif def='1' then
			def <= '0';
			adct <= adct+1;
			if (adct(6)='1' and emrw='1') or adct(7)='1' then	--for 64 or 128words, no change MAC
--			if adct(7)='1' then	--for 128words, to change MAC address
				pendf <= '1';
				acc <= '0';
				if emrw='1' then
					rwf <= '0';
					bf <= '1';
					cm <= "1000000000";	--EWDS
				end if; 
			else
				bf <= '1';
				if emrw='1' then
					cm(9 downto 7) <= "101";	--WRITE
					rwf <= '1';
					padd <= adct(6 downto 1)+1;
					cm(6 downto 0) <= adct(6 downto 0);
					if adct(0)='0' then
						mdib <= pdi(15 downto 8);
						bmdi <= pdi(7 downto 0);
					else
						mdib <= bmdi;
					end if;
				else 
					cm(9 downto 7) <= "110";	--READ
					padd <= adct(6 downto 1);
					cm(6 downto 0) <= adct(6 downto 0)+1;
					if adct(0)='0' then
						pdo(15 downto 8) <= mdob;
					else
						pdo(7 downto 0) <= mdob;
						prce <= '1';
					end if;
				end if; 
			end if;
		else
			bf <= '0';
			wait_ct <= wait_ct+1;
			if wait_ct=X"ffffff" then
				acc <= '0';
			end if;
		end if;
	elsif begf='1' and ddacc='0' and uph_on='0' and busy='0' then
		acc <= '1';
		def <= '0';
		vmef <= '1';
		bf <= '1';
		emrw <= rw;
		rwf <= '0';
		pendf <= '0';
		adct <= (others=>'0');
		padd <= (others=>'0');
		wait_ct <= (others=>'0');
		if rw='1' then
			cm <= "1001100000";	--EWEN
		else
			cm <= "1100000000";	--READ
		end if;
	elsif uph_on='1' then
		if ef='1' and def='0' then
			def <= '1';
			bf <= '0';
		elsif def='1' then
			def <= '0';
			adct <= adct+1;
			bf <= '1';
			if at_ct="00" then
				if adct(1 downto 0)="00" then
					atime(31 downto 24) <= mdob;
					cm(1 downto 0) <= "01";	--READ atime
				elsif adct(1 downto 0)="01" then
					atime(23 downto 16) <= mdob;
					cm(1 downto 0) <= "10";	--READ atime
				elsif adct(1 downto 0)="10" then
					atime(15 downto 8) <= mdob;
					cm(1 downto 0) <= "11";	--READ atime
				else
					atime <= (atime(31 downto 8)&mdob)+1;
					at_ct <= "01";
					cm <= "1001100000";	--EWEN
				end if;
			elsif at_ct="01" then
				rwf <= '1';
				cm(9 downto 2) <= "10111111";	--WRITE atime
				if adct(1 downto 0)="00" then
					mdib <= atime(7 downto 0);
					cm(1 downto 0) <= "11";	--WRITE atime
					if atime(7 downto 0)/=X"00" then
						at_ct <= "10";
					end if;
				elsif adct(1 downto 0)="01" then
					mdib <= atime(15 downto 8);
					cm(1 downto 0) <= "10";	--WRITE atime
					if atime(15 downto 0)/=X"0000" then
						at_ct <= "10";
					end if;
				elsif adct(1 downto 0)="10" then
					mdib <= atime(23 downto 16);
					cm(1 downto 0) <= "01";	--WRITE atime
					if atime(23 downto 0)/=X"000000" then
						at_ct <= "10";
					end if;
				else
					mdib <= atime(31 downto 24);
					cm(1 downto 0) <= "00";	--WRITE atime
					at_ct <= "10";
				end if;
			else
				uph_on <= '0';
				rwf <= '0';
				cm <= "1000000000";	--EWDS
			end if;
		else
			bf <= '0';
			wait_ct <= wait_ct+1;
			if wait_ct=X"ffffff" then
				acc <= '0';
			end if;
		end if;
	elsif up_1h='1' and begf='0' and busy='0' then
		up_1h <= '0';
		uph_on <= '1';
		at_ct <= "00";

		vmef <= '0';
		bf <= '1';
		def <= '0';
		rwf <= '0';
		adct <= "01111100";
		wait_ct <= (others=>'0');
		cm <= "1101111100";	--READ atime_H of EROM
	elsif locwb='1' then	--0x5a or 0x5b
		locwb <= '0';
		acc <= '1';
		vmef <= '0';
		bf <= '1';
		emrw <= lwtb;	--0x5a is read, 0x5b is write.
		rwf <= '0';
		adct <= (others=>'0');
		padd <= (others=>'0');
		wait_ct <= (others=>'0');
		if lwtb='1' then
			cm <= "1001100000";	--EWEN
		else
			cm <= "1100000000";	--READ EROM
		end if;
	else
		bf<='0';	vmef<='0';	pendf<='0';
	end if;

	if prce='1' and vmef='0' and acc='1' and padd(5 downto 4)="10" then	--add acc on 2008 1/4
			case padd(3 downto 0) is
			when	"0000"=>	ii_do(255 downto 240) <= pdo;
			when	"0001"=>	ii_do(239 downto 224) <= pdo;
			when	"0010"=>	ii_do(223 downto 208) <= pdo;
			when	"0011"=>	ii_do(207 downto 192) <= pdo;
			when	"0100"=>	ii_do(191 downto 176) <= pdo;
			when	"0101"=>	ii_do(175 downto 160) <= pdo;
			when	"0110"=>	ii_do(159 downto 144) <= pdo;
			when	"0111"=>	ii_do(143 downto 128) <= pdo;
			when	"1000"=>	ii_do(127 downto 112) <= pdo;
			when	"1001"=>	ii_do(111 downto 96) <= pdo;
			when	"1010"=>	ii_do(95 downto 80) <= pdo;
			when	"1011"=>	ii_do(79 downto 64) <= pdo;
			when	"1100"=>	ii_do(63 downto 48) <= pdo;
			when	"1101"=>	ii_do(47 downto 32) <= pdo;
			when	"1110"=>	ii_do(31 downto 16) <= pdo;
			when	others=>	ii_do(15 downto 0) <= pdo;
			end case;
	end if;

	if tim_1m_ct>=X"c34f" then	--1ms @50MHz
		tim_1m <= '1';
		tim_1m_ct <= (others=>'0');
	else
		tim_1m <= '0';
		tim_1m_ct <= tim_1m_ct+1;
	end if;
	if tim_1m='1' and up_1h='0' then
--		atime_ct <= (others=>'0');
		if hourct<"111011" then	-- 1 hour, 59
			hourct <= hourct+1;
--			up_1h <= '1';	--for test
		else
			hourct <= (others=>'0');
			up_1h <= '1';
		end if;
	end if;

end if;
end process;

end Behavioral;
